# AUTO-GENERATED FILE -- DO NOT EDIT


__package__ = None

def crypt(word, salt):
  """ crypt(word, salt) -> string
  word will usually be a user's password. salt is a 2-character string
  which will be used to select one of 4096 variations of DES. The characters
  in salt must be either ".", "/", or an alphanumeric character. Returns
  the hashed password as a string, which will be composed of characters from
  the same alphabet as the salt. """
  return ""

