class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
class Foo: pass
